"""
Remove overtime from your data
"""

from typing import Dict, Union
import pandas as pd
from ..errors import MissingColumns


def pop_overtime(
    df: pd.DataFrame,
    /,
    *,
    max_rounds_csgo: int = 30,
) -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
    """Remove events in overtime rounds

    Keywords:
        df (dataframe): Any individual channel with a "round" column.
        max_rounds_csgo (int): Default value: 30
            Max rounds of CS:GO for regular-time. This is 30 for normal
            matches and 16 for short matches.

    Returns:
        data (dataframe): The removed events.

    Examples:
        >>> pop_overtime(df)

        >>> df_ot = pop_overtime(df)

    """
    if "round" not in df.columns:
        raise MissingColumns("Cannot pop overtime", columns=["round"])
    overtime = df[df["round"] > max_rounds_csgo]
    index = df.index[~(df["round"] <= max_rounds_csgo)]
    df.drop(index, inplace=True)
    return overtime



# pylint: disable=missing-docstring
# pylint: disable=unused-import

import os
import pytest
from pureskillgg_dsdk import GameDsLoader, DsReaderFs
import pandas as pd
import structlog

from .pop_overtime import pop_overtime
from ..errors import MissingColumns, UnsupportedChannelStructure


def load_data(match_id):
    csds_reader = DsReaderFs(
        root_path=os.path.join("fixtures"),
        manifest_key=os.path.join("csds", "2022", "05", "15", match_id, "csds"),
        log=None,
    )
    csds_loader = GameDsLoader(reader=csds_reader, log=None)
    return csds_loader.get_channels([{"channel": "round_state"}, {"channel": "header"}])


def test_match_with_overtime():
    data = load_data("80e8dd2f-b876-4136-bfe5-6985bf2db179")
    original_length = data["round_state"].shape[0]
    data_overtime = {}
    data_overtime["round_state"] = pop_overtime(data["round_state"])
    assert isinstance(data["round_state"], pd.DataFrame)
    assert isinstance(data_overtime["round_state"], pd.DataFrame)
    assert data["round_state"].shape[0] < original_length
    assert data["round_state"].shape[0] > 0
    assert data_overtime["round_state"].shape[0] > 0
    assert (
        data_overtime["round_state"].shape[0] + data["round_state"].shape[0]
        == original_length
    )


def test_unsupported_channel_structure_error_usage():
    with pytest.raises(UnsupportedChannelStructure) as e_info:
        data = load_data("994a9fed-d4a5-4096-8088-93b422be5025")
        data_overtime = {}
        data_overtime["header"] = pop_overtime(data["header"])

    assert str(e_info.value) == "Cannot pop overtime: missing columns round"


def test_missing_columns_error_usage():
    with pytest.raises(MissingColumns) as e_info:
        data = load_data("994a9fed-d4a5-4096-8088-93b422be5025")
        data_overtime = {}
        data_overtime["header"] = pop_overtime(data["header"])

    assert e_info.value.columns == ["round"]
    assert str(e_info.value) == "Cannot pop overtime: missing columns round"


def test_short_match():
    data = load_data("9c9c6333-0eff-445f-9f18-6cb5138f944c")
    original_length = data["round_state"].shape[0]

    data_overtime = {}
    data_overtime["round_state"] = pop_overtime(data["round_state"])
    assert isinstance(data["round_state"], pd.DataFrame)
    assert isinstance(data["round_state"], pd.DataFrame)
    assert data["round_state"].shape[0] == original_length
    assert data_overtime["round_state"].shape[0] == 0


def test_regular_match():
    data = load_data("994a9fed-d4a5-4096-8088-93b422be5025")

    data_overtime = {}
    data_overtime["round_state"] = pop_overtime(data["round_state"])
    assert isinstance(data["round_state"], pd.DataFrame)
    assert isinstance(data_overtime["round_state"], pd.DataFrame)


test_unsupported_channel_structure_error_usage()
test_missing_columns_error_usage()



# pylint: disable=missing-docstring
# pylint: disable=unused-import

import copy
import os
import pytest
import dateutil.parser
from pureskillgg_dsdk import GameDsLoader, DsReaderFs

from .scrub_pii import scrub_csds_pii, SCRUB_CSDS_PII_CHANNEL_INSTRUCTIONS


def test_remove_pii():
    redacted = "redacted"
    csds_reader = DsReaderFs(
        root_path=os.path.join("fixtures"),
        manifest_key=os.path.join(
            "csds", "2022", "05", "15", "9c9c6333-0eff-445f-9f18-6cb5138f944c", "csds"
        ),
        log=None,
    )
    csds_loader = GameDsLoader(reader=csds_reader, log=None)
    data = csds_loader.get_channels(SCRUB_CSDS_PII_CHANNEL_INSTRUCTIONS)
    data["player_info"]["commends_teacher"] = 12345
    manifest = copy.deepcopy(csds_loader.manifest)

    manifest = scrub_csds_pii(manifest, data)
    assert data["header"]["sharecode"].iat[0] == redacted
    assert data["header"]["demo_id"].iat[0] == redacted
    assert data["player_personal"]["clan_tag"].iat[0] == redacted
    assert data["player_personal"]["name"].iat[0] == redacted
    steam_ids = data["player_personal"]["steam_id"]
    for steam_id in steam_ids:
        assert len(steam_id) == 1
    assert data["player_status"]["ping"].iat[100] == 0
    assert data["player_info"]["commends_teacher"].iat[0] == 101
    assert manifest["jobId"] == manifest["id"]
    assert manifest["sharecode"] == redacted
    assert manifest["demoId"] == redacted
    assert manifest["metadata"]["bucket"] == redacted
    date = dateutil.parser.isoparse(manifest["matchDate"])
    assert date.second == 0

    scrubbed_channels = [k["channel"] for k in SCRUB_CSDS_PII_CHANNEL_INSTRUCTIONS]
    for channel in manifest["channels"]:
        if channel["channel"] in scrubbed_channels:
            assert channel["redacted"] is True
        else:
            assert channel["redacted"] is False
    assert manifest["redacted"] is True



"""
Scrub PII from your data
"""
# pylint: disable=invalid-name
# pylint: disable=c-extension-no-member

from typing import Dict, Union
from numbers import Number
import dateutil.parser
import rapidjson

REDACTED = "redacted"

SCRUB_CSDS_PII_CHANNEL_INSTRUCTIONS = [
    {"channel": "player_name"},
    {"channel": "header"},
    {"channel": "player_personal"},
    {"channel": "player_info"},
    {"channel": "player_status"},
]


def scrub_csds_pii(manifest: Dict, data: Dict):
    """Remove events in overtime rounds

    Inputs:
        manifest (dict): CSDS manifest.
        data (dict): CSDS data dictionary.

    Returns:
        manifest (dict): CSDS manifest.
        The data is changed in place and not returned.
    """
    manifest = replace_job_id(manifest)
    manifest["sharecode"] = REDACTED
    manifest["demoId"] = REDACTED
    manifest["metadata"]["bucket"] = REDACTED
    manifest["matchDate"] = fix_date_precision(manifest["matchDate"])

    for i in range(len(manifest["channels"])):
        manifest["channels"][i]["redacted"] = False

    replace_if_exists(data, manifest, "header", "sharecode")
    replace_if_exists(data, manifest, "header", "demo_id")

    replace_if_exists(data, manifest, "player_name", "name_new")
    replace_if_exists(data, manifest, "player_name", "name_old")

    replace_if_exists(data, manifest, "player_personal", "name")
    replace_if_exists(data, manifest, "player_personal", "clan_tag")

    fix_steam_ids(data, manifest)

    fix_commends_and_wins(data, manifest)

    replace_if_exists(data, manifest, "player_status", "ping", replacement=0)

    return manifest


def data_exists(data, channel, column):
    """Check for a channel/column existing"""
    if channel in data:
        if column in data[channel].columns:
            return True
    return False


def replace_if_exists(
    data, manifest, channel, column, /, replacement: Union[str, bool, Number] = REDACTED
):
    """Replace a column with a certain value if it exists"""
    if data_exists(data, channel, column):
        data[channel][column] = replacement
        channel_index, column_index = get_manifest_indexes(manifest, channel, column)
        manifest["channels"][channel_index]["columns"][column_index][
            "origin"
        ] += "-redacted"
        manifest["channels"][channel_index]["redacted"] = True


def get_manifest_indexes(manifest, channel, column):
    """Get the indices for channel and column in the manifest"""
    channel_index = -1
    for x in range(len(manifest["channels"])):
        if manifest["channels"][x]["channel"] == channel:
            channel_index = x
            break
    if channel_index == -1:
        raise Exception(f"Channel does not exist: {channel}")
    column_index = -1
    for x in range(len(manifest["channels"][channel_index]["columns"])):
        if manifest["channels"][channel_index]["columns"][x]["name"] == column:
            column_index = x
    if column_index == -1:
        raise Exception(f"Column does not exist in channel {channel}: {column}")
    return channel_index, column_index


def fix_steam_ids(data, manifest):
    """Replace steam ids with A, B, C..."""
    if "player_personal" not in data:
        return
    pp = data["player_personal"]
    if "steam_id" not in pp.columns:
        return
    unique_ids = {}
    alphabet = "A B C D E F G H I J K L M N O P Q R S T U V W X Y Z".split()
    for i, steam_id in enumerate(pp["steam_id"].unique()):
        unique_ids[steam_id] = alphabet[i]
    pp["steam_id"] = pp["steam_id"].apply(lambda x: unique_ids[x])
    data["player_personal"] = pp
    channel_index, column_index = get_manifest_indexes(
        manifest, "player_personal", "steam_id"
    )
    manifest["channels"][channel_index]["columns"][column_index][
        "origin"
    ] += "-redacted"
    manifest["channels"][channel_index]["redacted"] = True


def fix_commends_and_wins(data, manifest):
    """Clear out too many commends and wins"""

    def cap_manifest_to_value(column, value=101):
        channel_index, column_index = get_manifest_indexes(
            manifest, "player_info", column
        )
        manifest["channels"][channel_index]["columns"][column_index][
            "origin"
        ] += "-capped"
        manifest["channels"][channel_index]["columns"][column_index][
            "comment"
        ] += f" Capped to {value}."
        manifest["channels"][channel_index]["redacted"] = True

    if "player_info" not in data:
        return
    pi = data["player_info"]
    columns = pi.columns
    if "wins" in columns:
        index = pi["wins"] > 2500
        pi.loc[index, "wins"] = 2501
        cap_manifest_to_value("wins", value=2501)
    if "commends_friendly" in columns:
        index = pi["commends_friendly"] > 100
        pi.loc[index, "commends_friendly"] = 101
        cap_manifest_to_value("commends_friendly")
    if "commends_leader" in columns:
        index = pi["commends_leader"] > 100
        pi.loc[index, "commends_leader"] = 101
        cap_manifest_to_value("commends_leader")
    if "commends_teacher" in columns:
        index = pi["commends_teacher"] > 100
        pi.loc[index, "commends_teacher"] = 101
        cap_manifest_to_value("commends_teacher")
    data["player_info"] = pi


def replace_job_id(manifest):
    """Replace job id in manifest with id"""
    job_id = manifest["jobId"]
    anon_id = manifest["id"]
    manifest_str = rapidjson.dumps(manifest)
    manifest_str = manifest_str.replace(job_id, anon_id)
    manifest = rapidjson.loads(manifest_str)
    manifest["redacted"] = True
    return manifest


def fix_date_precision(date, timespec="minutes"):
    """Set date to time precision"""
    dt = dateutil.parser.isoparse(date)
    return dt.isoformat(timespec=timespec)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pureskillgg_csgo_dsdk',
 'pureskillgg_csgo_dsdk.errors',
 'pureskillgg_csgo_dsdk.overtime',
 'pureskillgg_csgo_dsdk.scrubber']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.3.2,<2.0.0',
 'pureskillgg-dsdk>=0.30.0,<0.31.0',
 'structlog>=21.1.0,<22.0.0']

setup_kwargs = {
    'name': 'pureskillgg-csgo-dsdk',
    'version': '0.9.1',
    'description': 'Python Data Science Development Kit for CS:GO.',
    'long_description': 'PureSkill.gg CS:GO Data Science Development Kit\n===============================================\n\n|PyPI| |GitHub Actions|\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/pureskillgg-csgo-dsdk.svg\n   :target: https://pypi.python.org/pypi/pureskillgg-csgo-dsdk\n   :alt: PyPI\n.. |GitHub Actions| image:: https://github.com/pureskillgg/csgo-dsdk/workflows/main/badge.svg\n   :target: https://github.com/pureskillgg/csgo-dsdk/actions\n   :alt: GitHub Actions\n\nPython Data Science Development Kit for CS:GO.\n\nDescription\n-----------\n\nDo data science with CS:GO.\n\nInstallation\n------------\n\nThis package is registered on the `Python Package Index (PyPI)`_\nas pureskillgg-csgo-dsdk_.\n\nInstall it with\n\n::\n\n    $ poetry add pureskillgg-csgo-dsdk\n\n.. _pureskillgg-csgo-dsdk: https://pypi.python.org/pypi/pureskillgg-csgo-dsdk\n.. _Python Package Index (PyPI): https://pypi.python.org/\n\nDevelopment and Testing\n-----------------------\n\nQuickstart\n~~~~~~~~~~\n\n::\n\n    $ git clone https://github.com/pureskillgg/csgo-dsdk.git\n    $ git lfs install\n    $ git lfs pull\n    $ cd csgo-dsdk\n    $ poetry install\n\nRun each command below in a separate terminal window:\n\n::\n\n    $ make watch\n\nPrimary development tasks are defined in the `Makefile`.\n\nSource Code\n~~~~~~~~~~~\n\nThe `source code`_ is hosted on GitHub.\nClone the project with\n\n::\n\n    $ git clone https://github.com/pureskillgg/csgo-dsdk.git\n    $ git lfs install\n    $ git lfs pull\n\n.. _source code: https://github.com/pureskillgg/csgo-dsdk\n\nRequirements\n~~~~~~~~~~~~\n\nYou will need `Python 3`_ and Poetry_.\n\nInstall the development dependencies with\n\n::\n\n    $ poetry install\n\n.. _Poetry: https://poetry.eustace.io/\n.. _Python 3: https://www.python.org/\n\nTests\n~~~~~\n\nLint code with\n\n::\n\n    $ make lint\n\n\nRun tests with\n\n::\n\n    $ make test\n\nRun tests on changes with\n\n::\n\n    $ make watch\n\nPublishing\n~~~~~~~~~~\n\nUse the bump2version_ command to release a new version.\nPush the created git tag which will trigger a GitHub action.\n\n.. _bump2version: https://github.com/c4urself/bump2version\n\nPublishing may be triggered using on the web\nusing a `workflow_dispatch on GitHub Actions`_.\n\n.. _workflow_dispatch on GitHub Actions: https://github.com/pureskillgg/csgo-dsdk/actions?query=workflow%3Aversion\n\nGitHub Actions\n--------------\n\n*GitHub Actions should already be configured: this section is for reference only.*\n\nThe following repository secrets must be set on GitHub Actions.\n\n- ``PYPI_API_TOKEN``: API token for publishing on PyPI.\n\nThese must be set manually.\n\nSecrets for Optional GitHub Actions\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\nThe version and format GitHub actions\nrequire a user with write access to the repository\nincluding access to read and write packages.\nSet these additional secrets to enable the action:\n\n- ``GH_USER``: The GitHub user\'s username.\n- ``GH_TOKEN``: A personal access token for the user.\n- ``GIT_USER_NAME``: The name to set for Git commits.\n- ``GIT_USER_EMAIL``: The email to set for Git commits.\n- ``GPG_PRIVATE_KEY``: The `GPG private key`_.\n- ``GPG_PASSPHRASE``: The GPG key passphrase.\n\n.. _GPG private key: https://github.com/marketplace/actions/import-gpg#prerequisites\n\nContributing\n------------\n\nPlease submit and comment on bug reports and feature requests.\n\nTo submit a patch:\n\n1. Fork it (https://github.com/pureskillgg/csgo-dsdk/fork).\n2. Create your feature branch (`git checkout -b my-new-feature`).\n3. Make changes.\n4. Commit your changes (`git commit -am \'Add some feature\'`).\n5. Push to the branch (`git push origin my-new-feature`).\n6. Create a new Pull Request.\n\nLicense\n-------\n\nThis Python package is licensed under the MIT license.\n\nWarranty\n--------\n\nThis software is provided by the copyright holders and contributors "as is" and\nany express or implied warranties, including, but not limited to, the implied\nwarranties of merchantability and fitness for a particular purpose are\ndisclaimed. In no event shall the copyright holder or contributors be liable for\nany direct, indirect, incidental, special, exemplary, or consequential damages\n(including, but not limited to, procurement of substitute goods or services;\nloss of use, data, or profits; or business interruption) however caused and on\nany theory of liability, whether in contract, strict liability, or tort\n(including negligence or otherwise) arising in any way out of the use of this\nsoftware, even if advised of the possibility of such damage.\n',
    'author': 'PureSkill.gg',
    'author_email': 'contact@pureskill.gg',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pureskillgg/csgo-dsdk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""Exceptions for the csgo dsdk"""
from typing import List


class UnsupportedChannelStructure(Exception):
    """Exception raised for missing a column"""


class MissingColumns(UnsupportedChannelStructure):
    """Exception raised for missing a column"""

    def __init__(self, message: str, /, *, columns: List[str]):
        self.columns = columns
        self._message = message
        super().__init__(self._message)

    def __str__(self):
        return f"{self._message}: missing columns {', '.join(self.columns)}"



"""
PureSkill.gg CS:GO Data Science Development Kit.
"""
from .scrubber import scrub_csds_pii, SCRUB_CSDS_PII_CHANNEL_INSTRUCTIONS
from .overtime import pop_overtime
from .errors import MissingColumns
from .errors import UnsupportedChannelStructure
